using System;
using System.Reflection;
using DotGNU.Web.Services;
using System.IO;
public class Dll2Spec
{
	internal void ProcessMethod(Attribute attr,
				MethodInfo method)
	{
		Console.WriteLine("\t<method name=\"{0}\">",method.Name);
		Console.WriteLine("\t<help>");
		Console.WriteLine(((WebserviceAttribute)attr).DocText);
		Console.WriteLine("\t</help>");
		ParameterInfo []pinfos=method.GetParameters();
		Console.WriteLine("\t<retval type=\"{0}\"/>",
				method.ReturnType);
		Console.WriteLine("\t<params>");
		foreach(ParameterInfo pinfo in pinfos)
		{
			Console.WriteLine(
			"\t<parameter order=\"{0}\" type=\"{1}\" name=\"{2}\"/>",
				pinfo.Position,
				pinfo.ParameterType,pinfo.Name);
		}
		Console.WriteLine("\t</params>");
		Console.WriteLine("\t</method>");
	}
	internal WebserviceAttribute GetAttribute(MethodInfo method)
	{
		Attribute[] attrs=
			Attribute.GetCustomAttributes(method,typeof(WebserviceAttribute),
			false);
		if(attrs==null)
		{
			return null;
		}
		if(attrs.Length==1)
		{
			return (WebserviceAttribute)attrs[0];
		}
		return null;
	}
	internal void ProcessType(Type type)
	{
		Console.WriteLine("<type name=\"{0}\">",type.ToString());
		MethodInfo []methods=
			type.GetMethods(BindingFlags.Public | BindingFlags.Static);
		foreach(MethodInfo method in methods)
		{
			Attribute attr = GetAttribute(method);
			if(attr!=null)
			{
				ProcessMethod(attr,method);	
			}
		}
		Console.WriteLine("</type>");
	}
	internal void ProcessDll(String dllName)
	{
		String prefix=Path.GetFileNameWithoutExtension(dllName);
		Assembly ass=Assembly.Load(dllName);
		Type[] types=ass.GetTypes();
		Console.WriteLine("<assembly name=\"{0}\">",prefix);
		foreach(Type type in types)
		{
			if(type.ToString()!="<Module>")	ProcessType(type);
		}
		Console.WriteLine("</assembly>");
	}
	internal void ProcessArgs(String []args)
	{
		foreach(String arg in args)
		{
			if(arg.EndsWith(".dll"))
			{
				ProcessDll(arg);
			}
		}
	}
	public static void Main(String []args)
	{
		Dll2Spec dd=new Dll2Spec();
		if(args.Length<1)
		{
			dd.ProcessArgs(new String[] {"test.dll"});
		}
		else
		{
			dd.ProcessArgs(args);			
		}
	}
}
